package kr.co.vividnext.sodalive.content.comment

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.comment.QAudioContentComment.audioContentComment
import kr.co.vividnext.sodalive.fcm.QPushToken.pushToken
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
interface AudioContentCommentRepository : JpaRepository<AudioContentComment, Long>, AudioContentCommentQueryRepository

interface AudioContentCommentQueryRepository {
    fun findByContentId(
        cloudFrontHost: String,
        contentId: Long,
        memberId: Long,
        isContentCreator: Boolean,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem>

    fun totalCountCommentByContentId(contentId: Long, memberId: Long, isContentCreator: Boolean): Int
    fun commentReplyCountByAudioContentCommentId(commentId: Long): Int
    fun getAudioContentCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem>

    fun findPushTokenByContentIdAndCommentParentIdMyMemberId(
        contentId: Long,
        commentParentId: Long?,
        myMemberId: Long
    ): List<FindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse>
}

@Repository
class AudioContentCommentQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : AudioContentCommentQueryRepository {
    override fun findByContentId(
        cloudFrontHost: String,
        contentId: Long,
        memberId: Long,
        isContentCreator: Boolean,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem> {
        var where = audioContentComment.audioContent.id.eq(contentId)
            .and(audioContentComment.isActive.isTrue)
            .and(audioContentComment.parent.isNull)

        if (!isContentCreator) {
            where = where.and(
                audioContentComment.isSecret.isFalse
                    .or(audioContentComment.member.id.eq(memberId))
            )
        }

        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                audioContentComment.createdAt,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y.%m.%d %W %h:%i %p"
        )

        return queryFactory
            .select(
                QGetAudioContentCommentListItem(
                    audioContentComment.id,
                    audioContentComment.member.id,
                    audioContentComment.member.nickname,
                    audioContentComment.member.profileImage.prepend("/").prepend(cloudFrontHost),
                    audioContentComment.comment,
                    audioContentComment.isSecret,
                    audioContentComment.donationCan.coalesce(0),
                    formattedDate,
                    Expressions.constant(0)
                )
            )
            .from(audioContentComment)
            .innerJoin(audioContentComment.audioContent, audioContent)
            .innerJoin(audioContentComment.member, member)
            .where(where)
            .offset(offset)
            .limit(limit.toLong())
            .orderBy(audioContentComment.createdAt.desc())
            .fetch()
            .map {
                it.replyCount = commentReplyCountByAudioContentCommentId(it.id)
                it
            }
    }

    override fun totalCountCommentByContentId(contentId: Long, memberId: Long, isContentCreator: Boolean): Int {
        var where = audioContentComment.audioContent.id.eq(contentId)
            .and(audioContentComment.isActive.isTrue)
            .and(audioContentComment.parent.isNull)

        if (!isContentCreator) {
            where = where.and(
                audioContentComment.isSecret.isFalse
                    .or(audioContentComment.member.id.eq(memberId))
            )
        }

        return queryFactory
            .select(audioContentComment.id)
            .from(audioContentComment)
            .innerJoin(audioContentComment.audioContent, audioContent)
            .innerJoin(audioContentComment.member, member)
            .where(where)
            .fetch()
            .size
    }

    override fun commentReplyCountByAudioContentCommentId(commentId: Long): Int {
        return queryFactory.select(audioContentComment.id)
            .from(audioContentComment)
            .where(
                audioContentComment.parent.isNotNull
                    .and(audioContentComment.parent.id.eq(commentId))
                    .and(audioContentComment.isActive.isTrue)
            )
            .fetch()
            .size
    }

    override fun getAudioContentCommentReplyList(
        cloudFrontHost: String,
        commentId: Long,
        timezone: String,
        offset: Long,
        limit: Int
    ): List<GetAudioContentCommentListItem> {
        val formattedDate = Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                audioContentComment.createdAt,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y.%m.%d %W %h:%i %p"
        )

        return queryFactory
            .select(
                QGetAudioContentCommentListItem(
                    audioContentComment.id,
                    audioContentComment.member.id,
                    audioContentComment.member.nickname,
                    audioContentComment.member.profileImage.prepend("/").prepend(cloudFrontHost),
                    audioContentComment.comment,
                    audioContentComment.isSecret,
                    audioContentComment.donationCan.coalesce(0),
                    formattedDate,
                    Expressions.constant(0)
                )
            )
            .from(audioContentComment)
            .innerJoin(audioContentComment.member, member)
            .where(
                audioContentComment.parent.isNotNull
                    .and(audioContentComment.parent.id.eq(commentId))
                    .and(audioContentComment.isActive.isTrue)
            )
            .offset(offset)
            .limit(limit.toLong())
            .orderBy(audioContentComment.createdAt.desc())
            .fetch()
    }

    override fun findPushTokenByContentIdAndCommentParentIdMyMemberId(
        contentId: Long,
        commentParentId: Long?,
        myMemberId: Long
    ): List<FindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse> {
        var where = audioContent.id.eq(contentId)
            .and(member.id.ne(myMemberId))

        if (commentParentId != null) {
            where = where.and(
                audioContentComment.parent.id.eq(commentParentId)
                    .or(audioContentComment.id.eq(commentParentId))
            )
                .and(audioContentComment.isActive.isTrue)
        }

        val response = if (commentParentId != null) {
            queryFactory
                .select(
                    QFindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse(
                        pushToken.token,
                        pushToken.deviceType
                    )
                )
                .from(audioContentComment)
                .innerJoin(audioContentComment.audioContent, audioContent)
                .innerJoin(audioContentComment.member, member)
                .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
                .where(where)
                .fetch()
        } else {
            queryFactory
                .select(
                    QFindPushTokenByContentIdAndCommentParentIdMyMemberIdResponse(
                        pushToken.token,
                        pushToken.deviceType
                    )
                )
                .from(audioContent)
                .innerJoin(audioContent.member, member)
                .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
                .where(where)
                .fetch()
        }

        return response
    }
}
