package kr.co.vividnext.sodalive.content.comment

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.order.OrderRepository
import kr.co.vividnext.sodalive.fcm.FcmEvent
import kr.co.vividnext.sodalive.fcm.FcmEventType
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.context.ApplicationEventPublisher
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional

@Service
@Transactional(readOnly = true)
class AudioContentCommentService(
    private val repository: AudioContentCommentRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val audioContentRepository: AudioContentRepository,
    private val applicationEventPublisher: ApplicationEventPublisher,
    private val orderRepository: OrderRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    @Transactional
    fun registerComment(
        member: Member,
        comment: String,
        audioContentId: Long,
        parentId: Long? = null,
        isSecret: Boolean = false
    ): Long {
        val audioContent = audioContentRepository.findByIdOrNull(id = audioContentId)
            ?: throw SodaException("잘못된 콘텐츠 입니다.\n다시 시도해 주세요.")

        val creator = audioContent.member!!
        val isBlocked = blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = creator.id!!)
        if (isBlocked) throw SodaException("${creator.nickname}님의 요청으로 댓글쓰기가 제한됩니다.")

        val (isExistsAudioContent, _) = orderRepository.isExistOrderedAndOrderType(
            memberId = member.id!!,
            contentId = audioContent.id!!
        )

        if (isSecret && !isExistsAudioContent) {
            throw SodaException("콘텐츠 구매 후 비밀댓글을 등록할 수 있습니다.")
        }

        val audioContentComment = AudioContentComment(comment = comment, isSecret = isSecret)
        audioContentComment.audioContent = audioContent
        audioContentComment.member = member

        val parent = if (parentId != null) {
            repository.findByIdOrNull(id = parentId)
        } else {
            null
        }

        if (parent != null) {
            audioContentComment.parent = parent
        }

        val savedContentComment = repository.save(audioContentComment)

        applicationEventPublisher.publishEvent(
            FcmEvent(
                type = FcmEventType.CREATE_CONTENT_COMMENT,
                title = if (parent != null) {
                    parent.member!!.nickname
                } else {
                    member.nickname
                },
                message = if (parent != null) {
                    "댓글에 답글을 달았습니다.: ${audioContent.title}"
                } else {
                    "콘텐츠에 댓글을 달았습니다.: ${audioContent.title}"
                },
                contentId = audioContentId,
                commentParentId = parentId,
                myMemberId = member.id
            )
        )

        return savedContentComment.id!!
    }

    @Transactional
    fun modifyComment(request: ModifyCommentRequest, member: Member) {
        val audioContentComment = repository.findByIdOrNull(request.commentId)
            ?: throw SodaException("잘못된 접근 입니다.\n확인 후 다시 시도해 주세요.")

        if (audioContentComment.member!!.id!! == member.id!!) {
            if (request.comment != null) {
                audioContentComment.comment = request.comment
            }
        }

        if (
            audioContentComment.member!!.id!! == member.id!! ||
            audioContentComment.audioContent!!.member!!.id!! == member.id!!
        ) {
            if (request.isActive != null) {
                audioContentComment.isActive = request.isActive
            }
        }
    }

    fun getCommentList(
        audioContentId: Long,
        memberId: Long,
        timezone: String,
        pageable: Pageable
    ): GetAudioContentCommentListResponse {
        val isContentCreator = audioContentRepository.isContentCreator(audioContentId, memberId)
        val commentList = repository.findByContentId(
            cloudFrontHost = cloudFrontHost,
            contentId = audioContentId,
            memberId = memberId,
            isContentCreator = isContentCreator,
            timezone = timezone,
            offset = pageable.offset,
            limit = pageable.pageSize
        )
        val totalCount = repository.totalCountCommentByContentId(
            contentId = audioContentId,
            memberId = memberId,
            isContentCreator = isContentCreator
        )

        return GetAudioContentCommentListResponse(totalCount, commentList)
    }

    fun getCommentReplyList(
        commentId: Long,
        timezone: String,
        pageable: Pageable
    ): GetAudioContentCommentListResponse {
        val commentList = repository.getAudioContentCommentReplyList(
            cloudFrontHost = cloudFrontHost,
            commentId = commentId,
            timezone = timezone,
            offset = pageable.offset,
            limit = pageable.pageSize
        )
        val totalCount = repository.commentReplyCountByAudioContentCommentId(commentId)

        return GetAudioContentCommentListResponse(totalCount, commentList)
    }
}
