package kr.co.vividnext.sodalive.content.series

import com.querydsl.core.types.dsl.Expressions
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.content.series.genre.QSeriesGenre.seriesGenre
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.hashtag.QHashTag.hashTag
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCuration.audioContentCuration
import kr.co.vividnext.sodalive.content.main.curation.QAudioContentCurationItem.audioContentCurationItem
import kr.co.vividnext.sodalive.content.series.content.GetSeriesContentMinMaxPriceResponse
import kr.co.vividnext.sodalive.content.series.content.QGetSeriesContentMinMaxPriceResponse
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeriesContent.seriesContent
import kr.co.vividnext.sodalive.creator.admin.content.series.Series
import kr.co.vividnext.sodalive.creator.admin.content.series.keyword.QSeriesKeyword.seriesKeyword
import kr.co.vividnext.sodalive.member.MemberRole
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import org.springframework.data.jpa.repository.JpaRepository

interface ContentSeriesRepository : JpaRepository<Series, Long>, ContentSeriesQueryRepository

interface ContentSeriesQueryRepository {
    fun getSeriesTotalCount(creatorId: Long, isAuth: Boolean, contentType: ContentType): Int
    fun getSeriesList(
        imageHost: String,
        creatorId: Long,
        isAuth: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<Series>

    fun getSeriesDetail(seriesId: Long, isAuth: Boolean, contentType: ContentType): Series?
    fun getKeywordList(seriesId: Long): List<String>
    fun getSeriesContentMinMaxPrice(seriesId: Long): GetSeriesContentMinMaxPriceResponse
    fun getRecommendSeriesList(isAuth: Boolean, contentType: ContentType, limit: Long): List<Series>
    fun getOriginalAudioDramaList(
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20
    ): List<Series>

    fun getOriginalAudioDramaTotalCount(memberId: Long, isAdult: Boolean, contentType: ContentType): Int
    fun getGenreList(isAdult: Boolean, memberId: Long, contentType: ContentType): List<GetSeriesGenreListResponse>
    fun findByCurationId(curationId: Long, memberId: Long, isAdult: Boolean, contentType: ContentType): List<Series>
}

class ContentSeriesQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory
) : ContentSeriesQueryRepository {
    override fun getSeriesTotalCount(creatorId: Long, isAuth: Boolean, contentType: ContentType): Int {
        var where = series.member.id.eq(creatorId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(series.id)
            .from(series)
            .innerJoin(series.member, member)
            .where(where)
            .fetch()
            .size
    }

    override fun getSeriesList(
        imageHost: String,
        creatorId: Long,
        isAuth: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<Series> {
        var where = series.member.id.eq(creatorId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .selectFrom(series)
            .innerJoin(series.member, member)
            .where(where)
            .orderBy(series.orders.asc(), series.createdAt.asc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getSeriesDetail(seriesId: Long, isAuth: Boolean, contentType: ContentType): Series? {
        var where = series.id.eq(seriesId)
            .and(series.isActive.isTrue)

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .selectFrom(series)
            .innerJoin(series.member, member)
            .where(where)
            .fetchFirst()
    }

    override fun getKeywordList(seriesId: Long): List<String> {
        return queryFactory
            .select(hashTag.tag)
            .from(series)
            .innerJoin(series.keywordList, seriesKeyword)
            .innerJoin(seriesKeyword.keyword, hashTag)
            .where(series.id.eq(seriesId))
            .fetch()
    }

    override fun getSeriesContentMinMaxPrice(seriesId: Long): GetSeriesContentMinMaxPriceResponse {
        return queryFactory
            .select(
                QGetSeriesContentMinMaxPriceResponse(
                    audioContent.price.min(),
                    audioContent.price.max()
                )
            )
            .from(series)
            .innerJoin(series.contentList, seriesContent)
            .innerJoin(seriesContent.content, audioContent)
            .where(series.id.eq(seriesId))
            .fetchFirst()
    }

    override fun getRecommendSeriesList(isAuth: Boolean, contentType: ContentType, limit: Long): List<Series> {
        var where = series.isActive.isTrue

        if (!isAuth) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .selectFrom(series)
            .where(where)
            .orderBy(Expressions.numberTemplate(Double::class.java, "function('rand')").asc())
            .offset(0)
            .limit(limit)
            .fetch()
    }

    override fun getOriginalAudioDramaList(
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long,
        limit: Long
    ): List<Series> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isOriginal.isTrue
            .and(series.isActive.isTrue)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .selectFrom(series)
            .innerJoin(series.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .orderBy(series.id.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    override fun getOriginalAudioDramaTotalCount(memberId: Long, isAdult: Boolean, contentType: ContentType): Int {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isOriginal.isTrue
            .and(series.isActive.isTrue)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(series.id)
            .from(series)
            .innerJoin(series.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .fetch()
            .size
    }

    override fun getGenreList(
        isAdult: Boolean,
        memberId: Long,
        contentType: ContentType
    ): List<GetSeriesGenreListResponse> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = seriesGenre.isActive.isTrue
            .and(series.isActive.isTrue)
            .and(audioContent.isActive.isTrue)
            .and(member.isActive.isTrue)
            .and(member.isNotNull)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContent.duration.isNotNull)
            .and(audioContent.limited.isNull)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(seriesGenre.isAdult.isFalse)
                .and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(QGetSeriesGenreListResponse(seriesGenre.id, seriesGenre.genre))
            .from(seriesContent)
            .innerJoin(seriesContent.series, series)
            .innerJoin(series.genre, seriesGenre)
            .innerJoin(seriesContent.content, audioContent)
            .innerJoin(series.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .groupBy(seriesGenre.id)
            .orderBy(seriesGenre.orders.asc())
            .fetch()
    }

    override fun findByCurationId(
        curationId: Long,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<Series> {
        val blockMemberCondition = blockMember.member.id.eq(member.id)
            .and(blockMember.isActive.isTrue)
            .and(blockMember.blockedMember.id.eq(memberId))

        var where = series.isActive.isTrue
            .and(member.isActive.isTrue)
            .and(member.role.eq(MemberRole.CREATOR))
            .and(audioContentCuration.id.eq(curationId))
            .and(audioContentCurationItem.isActive.isTrue)
            .and(blockMember.id.isNull)

        if (!isAdult) {
            where = where.and(series.isAdult.isFalse)
        } else {
            if (contentType != ContentType.ALL) {
                where = where.and(
                    series.member.isNull.or(
                        series.member.auth.gender.eq(
                            if (contentType == ContentType.MALE) {
                                0
                            } else {
                                1
                            }
                        )
                    )
                )
            }
        }

        return queryFactory
            .select(series)
            .from(audioContentCurationItem)
            .innerJoin(audioContentCurationItem.curation, audioContentCuration)
            .innerJoin(audioContentCurationItem.series, series)
            .innerJoin(series.member, member)
            .leftJoin(blockMember).on(blockMemberCondition)
            .where(where)
            .orderBy(audioContentCurationItem.orders.asc())
            .fetch()
    }
}
