package kr.co.vividnext.sodalive.content.series

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.order.OrderRepository
import kr.co.vividnext.sodalive.content.order.OrderType
import kr.co.vividnext.sodalive.content.series.content.ContentSeriesContentRepository
import kr.co.vividnext.sodalive.content.series.content.GetSeriesContentListResponse
import kr.co.vividnext.sodalive.creator.admin.content.series.Series
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesPublishedDaysOfWeek
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesSortType
import kr.co.vividnext.sodalive.creator.admin.content.series.SeriesState
import kr.co.vividnext.sodalive.explorer.ExplorerQueryRepository
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import java.time.LocalDateTime
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class ContentSeriesService(
    private val repository: ContentSeriesRepository,
    private val orderRepository: OrderRepository,
    private val blockMemberRepository: BlockMemberRepository,
    private val explorerQueryRepository: ExplorerQueryRepository,
    private val seriesContentRepository: ContentSeriesContentRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val coverImageHost: String
) {
    fun getOriginalAudioDramaTotalCount(memberId: Long, isAdult: Boolean, contentType: ContentType): Int {
        return repository.getOriginalAudioDramaTotalCount(memberId, isAdult, contentType)
    }

    fun getOriginalAudioDramaList(
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType,
        offset: Long = 0,
        limit: Long = 20
    ): List<GetSeriesListResponse.SeriesListItem> {
        val originalAudioDramaList = repository.getOriginalAudioDramaList(memberId, isAdult, contentType, offset, limit)
        return seriesToSeriesListItem(originalAudioDramaList, isAdult, contentType)
    }

    fun getGenreList(memberId: Long, isAdult: Boolean, contentType: ContentType): List<GetSeriesGenreListResponse> {
        return repository.getGenreList(memberId = memberId, isAdult = isAdult, contentType = contentType)
    }

    fun getSeriesList(
        creatorId: Long,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        sortType: SeriesSortType = SeriesSortType.NEWEST,
        offset: Long = 0,
        limit: Long = 10
    ): GetSeriesListResponse {
        val isAuth = member.auth != null && isAdultContentVisible

        val totalCount = repository.getSeriesTotalCount(
            creatorId = creatorId,
            isAuth = isAuth,
            contentType = contentType
        )
        val rawItems = repository.getSeriesList(
            imageHost = coverImageHost,
            creatorId = creatorId,
            isAuth = isAuth,
            contentType = contentType,
            offset = offset,
            limit = limit
        ).filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.member!!.id!!) }

        val items = seriesToSeriesListItem(seriesList = rawItems, isAdult = isAuth, contentType = contentType)
        return GetSeriesListResponse(totalCount, items)
    }

    fun getSeriesDetail(
        seriesId: Long,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): GetSeriesDetailResponse {
        val series = repository.getSeriesDetail(
            seriesId = seriesId,
            isAuth = member.auth != null && isAdultContentVisible,
            contentType = contentType
        ) ?: throw SodaException("잘못된 시리즈 입니다.\n다시 시도해 주세요")

        val isBlocked = blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = series.member!!.id!!)
        if (isBlocked) {
            throw SodaException("잘못된 시리즈 입니다.\n다시 시도해 주세요")
        }

        val creatorFollowing = explorerQueryRepository.getCreatorFollowing(
            creatorId = series.member!!.id!!,
            memberId = member.id!!
        )

        val keywordList = repository.getKeywordList(seriesId = seriesId)
            .filter { it.isNotBlank() }

        val minMaxPrice = repository.getSeriesContentMinMaxPrice(seriesId = seriesId)
        val minPrice = minMaxPrice.minPrice
        val maxPrice = minMaxPrice.maxPrice
        val rentalMinPrice = (minMaxPrice.minPrice * 0.7).toInt()
        val rentalMaxPrice = (minMaxPrice.maxPrice * 0.7).toInt()

        val seriesContentList = getSeriesContentList(
            seriesId = seriesId,
            isAdultContentVisible = isAdultContentVisible,
            contentType = contentType,
            member = member,
            sortType = SeriesSortType.NEWEST,
            offset = 0,
            limit = 5
        )

        val dateTimeFormatter = DateTimeFormatter.ofPattern("yyyy.MM.dd")
        return GetSeriesDetailResponse(
            seriesId = seriesId,
            title = series.title,
            coverImage = "$coverImageHost/${series.coverImage}",
            introduction = series.introduction,
            genre = series.genre!!.genre,
            isAdult = series.isAdult,
            writer = series.writer,
            studio = series.studio,
            publishedDate = series.createdAt!!
                .atZone(ZoneId.of("UTC"))
                .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                .toLocalDateTime()
                .format(dateTimeFormatter),
            creator = GetSeriesDetailResponse.GetSeriesDetailCreator(
                creatorId = series.member!!.id!!,
                nickname = series.member!!.nickname,
                profileImage = "$coverImageHost/${series.member!!.profileImage}",
                isFollow = creatorFollowing?.isFollow ?: false,
                isNotify = creatorFollowing?.isNotify ?: false
            ),
            rentalMinPrice = rentalMinPrice,
            rentalMaxPrice = rentalMaxPrice,
            rentalPeriod = 15,
            minPrice = minPrice,
            maxPrice = maxPrice,
            keywordList = keywordList,
            publishedDaysOfWeek = publishedDaysOfWeekText(series.publishedDaysOfWeek),
            contentList = seriesContentList.items,
            contentCount = seriesContentList.totalCount
        )
    }

    fun getSeriesContentList(
        seriesId: Long,
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member,
        sortType: SeriesSortType,
        offset: Long,
        limit: Long
    ): GetSeriesContentListResponse {
        val isAdult = member.auth != null && isAdultContentVisible

        val totalCount = seriesContentRepository.getContentCount(seriesId, isAdult = isAdult, contentType = contentType)
        val contentList = seriesContentRepository.getContentList(
            seriesId = seriesId,
            isAdult = isAdult,
            contentType = contentType,
            imageHost = coverImageHost,
            sortType = sortType,
            offset = offset,
            limit = limit
        )
            .map {
                val (isExistsAudioContent, orderType) = orderRepository.isExistOrderedAndOrderType(
                    memberId = member.id!!,
                    contentId = it.contentId
                )

                if (isExistsAudioContent) {
                    if (orderType == OrderType.RENTAL) {
                        it.isRented = true
                    } else {
                        it.isOwned = true
                    }
                }

                it
            }

        return GetSeriesContentListResponse(totalCount, contentList)
    }

    fun getRecommendSeriesList(
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): List<GetSeriesListResponse.SeriesListItem> {
        val isAuth = member.auth != null && isAdultContentVisible
        val seriesList = repository.getRecommendSeriesList(
            isAuth = isAuth,
            contentType = contentType,
            limit = 10
        ).filter { !blockMemberRepository.isBlocked(blockedMemberId = member.id!!, memberId = it.member!!.id!!) }

        return seriesToSeriesListItem(seriesList = seriesList, isAdult = isAuth, contentType = contentType)
    }

    fun fetchSeriesByCurationId(
        curationId: Long,
        memberId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetSeriesListResponse.SeriesListItem> {
        val seriesList = repository.findByCurationId(
            curationId = curationId,
            memberId = memberId,
            isAdult = isAdult,
            contentType = contentType
        )
        return seriesToSeriesListItem(seriesList, isAdult, contentType)
    }

    private fun seriesToSeriesListItem(
        seriesList: List<Series>,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetSeriesListResponse.SeriesListItem> {
        return seriesList
            .map {
                GetSeriesListResponse.SeriesListItem(
                    seriesId = it.id!!,
                    title = it.title,
                    coverImage = "$coverImageHost/${it.coverImage!!}",
                    publishedDaysOfWeek = publishedDaysOfWeekText(it.publishedDaysOfWeek),
                    isComplete = it.state == SeriesState.COMPLETE,
                    creator = GetSeriesListResponse.SeriesListItemCreator(
                        creatorId = it.member!!.id!!,
                        nickname = it.member!!.nickname,
                        profileImage = "$coverImageHost/${it.member!!.profileImage!!}"
                    )
                )
            }
            .map {
                it.numberOfContent = seriesContentRepository.getContentCount(
                    seriesId = it.seriesId,
                    isAdult = isAdult,
                    contentType = contentType
                )

                it
            }
            .filter {
                it.numberOfContent > 0
            }
            .map {
                val nowDateTime = LocalDateTime.now()

                it.isNew = seriesContentRepository.isNewContent(
                    seriesId = it.seriesId,
                    isAdult = isAdult,
                    fromDate = nowDateTime.minusDays(7),
                    nowDate = nowDateTime
                )

                it
            }
    }

    private fun publishedDaysOfWeekText(publishedDaysOfWeek: Set<SeriesPublishedDaysOfWeek>): String {
        val dayOfWeekText = publishedDaysOfWeek.toList().sortedBy { it.ordinal }
            .map {
                when (it) {
                    SeriesPublishedDaysOfWeek.SUN -> "일"
                    SeriesPublishedDaysOfWeek.MON -> "월"
                    SeriesPublishedDaysOfWeek.TUE -> "화"
                    SeriesPublishedDaysOfWeek.WED -> "수"
                    SeriesPublishedDaysOfWeek.THU -> "목"
                    SeriesPublishedDaysOfWeek.FRI -> "금"
                    SeriesPublishedDaysOfWeek.SAT -> "토"
                    SeriesPublishedDaysOfWeek.RANDOM -> "랜덤"
                }
            }
            .joinToString(", ") { it }

        return if (publishedDaysOfWeek.contains(SeriesPublishedDaysOfWeek.RANDOM)) {
            dayOfWeekText
        } else if (publishedDaysOfWeek.size < 7) {
            "매주 $dayOfWeekText"
        } else {
            "매일"
        }
    }
}
